package com.search.supersearch.controller;

import cn.hutool.core.collection.CollUtil;
import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.core.CountRequest;
import co.elastic.clients.elasticsearch.core.CountResponse;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.elasticsearch.core.search.Hit;
import co.elastic.clients.elasticsearch.core.search.HitsMetadata;
import com.google.common.collect.Lists;
import com.search.supersearch.common.R;
import com.search.supersearch.model.dto.SearchDTO;
import com.search.supersearch.model.pojo.PlatomixTemplateCate;
import com.search.supersearch.model.vo.DocsIndexCountTreeVO;
import com.search.supersearch.model.vo.DocsIndexVO;
import com.search.supersearch.service.DocsService;
import com.search.supersearch.service.PlatomixTemplateCateService;
import io.swagger.annotations.Api;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/docs/v3")
@Slf4j
@Api(value = "文档搜索", tags = "文档搜索")
public class DocsV3Controller {

    @Resource
    private ElasticsearchClient elasticsearchClient;
    @Resource
    private DocsService docsService;
    @Resource
    private PlatomixTemplateCateService platomixTemplateCateService;

    @PostMapping("/search")
    @SneakyThrows
    public R<DocsIndexVO> searchV2(@RequestBody SearchDTO dto, HttpServletRequest request) {
        log.info("com.search.supersearch.controller.docs.DocsV3Controller.searchV2 开始执行={}", System.currentTimeMillis());

        String keyword = dto.getKeyword();
        if (CollUtil.isEmpty(dto.getIndex())) {
            return R.fail("参数为空!");
        }

        if (dto.getSize() == null) {
            dto.setSize(3);
        }
        SearchResponse<Map> search = docsService.searchHitv2(keyword, dto.getMatch(), dto.getSort(), dto.getIndex(), dto.getSize(), dto.getSearchAfter());
        if (search == null) {
            return R.data(new DocsIndexVO());
        }
        DocsIndexVO vo = getDocsIndexVO(search.hits(), "", "");
        log.info("com.search.supersearch.controller.docs.DocsV3Controller.searchV2 执行结束={}", System.currentTimeMillis());
        return R.data(vo);
    }

    @PostMapping("/searchAll")
    @SneakyThrows
    public R<List<DocsIndexVO>> searchAllV2(@RequestBody SearchDTO dto, HttpServletRequest request) {
        log.info("com.search.supersearch.controller.docs.DocsV3Controller.searchAllV2 开始执行={}", System.currentTimeMillis());

        String keyword = dto.getKeyword();

        if (dto.getSize() == null) {
            dto.setSize(3);
        }
        ArrayList<DocsIndexVO> vos = Lists.newArrayList();
        List<String> hideType = dto.getHideType();
        List<PlatomixTemplateCate> cates = platomixTemplateCateService.lambdaQuery().list();
//        List<String> index = cates.stream().map(cate->cate.getId().toString()).collect(Collectors.toList());

        for (PlatomixTemplateCate cate : cates) {
            SearchResponse<Map> search = docsService.searchHitv2(keyword, dto.getMatch(), dto.getSort(), Lists.newArrayList(cate.getId().toString()), dto.getSize(), dto.getSearchAfter());
            if (search == null) {
                continue;
            }
            HitsMetadata<Map> hits = search.hits();
            if (hits.total() == null || hits.total().value() == 0) {
                continue;
            }
            DocsIndexVO vo = getDocsIndexVO(hits, cate.getCreateId(), cate.getName());
            vos.add(vo);
        }


        log.info("com.search.supersearch.controller.docs.DocsV3Controller.searchAllV2 执行结束={}", System.currentTimeMillis());

        return R.data(vos);
    }

    @PostMapping("/searchCount")
    @SneakyThrows
    public R<List<DocsIndexCountTreeVO>> searchCountV2(@RequestBody SearchDTO dto, HttpServletRequest request) {
        log.info("com.search.supersearch.controller.docs.DocsV3Controller.searchCountV2 开始执行={}", System.currentTimeMillis());

        List<DocsIndexCountTreeVO> vos = new ArrayList<>();
        List<PlatomixTemplateCate> cates = platomixTemplateCateService.lambdaQuery().list();

        for (PlatomixTemplateCate cate : cates) {
            vos.add(extracted(dto, cate));
        }
        for (DocsIndexCountTreeVO vo : vos) {
            vo.setTotal(vo.getTotal() + recursionTotal(vo.getInfo()));
        }
        log.info("com.search.supersearch.controller.docs.DocsV3Controller.searchCountV2 执行结束={}", System.currentTimeMillis());
        return R.data(vos);
    }


    private Long recursionTotal(List<DocsIndexCountTreeVO> info) {
        long total = 0L;
        if (info == null) {
            return total;
        }
        for (DocsIndexCountTreeVO vo : info) {
            long voTotal = vo.getTotal() + recursionTotal(vo.getInfo());
            total += voTotal;
            vo.setTotal(voTotal);
        }
        return total;
    }

    private DocsIndexCountTreeVO extracted(SearchDTO dto, PlatomixTemplateCate cate) {

        DocsIndexCountTreeVO vo = new DocsIndexCountTreeVO();
        vo.setType(cate.getId().toString());
        vo.setName(cate.getName());
//        vo.setInfo();
        CountResponse count = null;
        try {
            count = elasticsearchClient.count(c -> {
                CountRequest.Builder countRequestBuilder = c.index(cate.getId().toString());
                docsService.matchCriteriaCountRequest(dto.getMatch(), Lists.newArrayList(cate.getId().toString()), dto.getKeyword(), countRequestBuilder);
                return countRequestBuilder;
            });
            vo.setTotal(count.count());
            log.info("成功查询dataCategory.getCategoryId()={},index={}", cate.getId(), cate.getName());
        } catch (Exception e) {
            vo.setTotal(0L);
            log.info("失败查询dataCategory.getCategoryId()={},index={}", cate.getId(), cate.getName());
        }
        return vo;
    }

    private DocsIndexVO getDocsIndexVO(HitsMetadata<Map> hits, String type, String name) {
        ArrayList<Object> list = Lists.newArrayList();
        List<String> searchAfter = Lists.newArrayList();


        for (Hit<Map> docsIndexHit : hits.hits()) {
            DocsIndexVO.DocsIndexInfo info = new DocsIndexVO.DocsIndexInfo();
            Map source = docsIndexHit.source();
            assert source != null;
            String es_content = (String) source.get("es_data_simple_content");
            if (StringUtils.isNotEmpty(es_content)) {
                source.put("es_data_content", es_content);

                if (StringUtils.isNotEmpty(es_content) && es_content.length() > 500) {
                    source.put("es_data_content", es_content.substring(0, 500));
                }

            }

            String path = (String) source.get("es_file_path");
            if (StringUtils.isNotBlank(path)) {
                source.put("es_file_path", path.substring(path.lastIndexOf("/") + 1));

            }
            info.setSource(source);
            info.setHighlight(docsIndexHit.highlight());
            List<String> collect = docsIndexHit.sort().stream().map(v -> v._get() + "").collect(Collectors.toList());
            info.setSearchAfter(collect);
            searchAfter = collect;
            list.add(info);
        }

        DocsIndexVO vo = new DocsIndexVO();
        vo.setTotal(hits.total().value());
        vo.setData(list);
        vo.setType(type);
        vo.setName(name);
        return vo;
    }

}
